package com.platform.yijia.controller;
import com.alibaba.fastjson.JSONObject;
import com.google.gson.Gson;
import com.platform.yijia.param.request.PayOrderRequest;
import com.platform.yijia.pojo.AppUserInfo;
import com.platform.yijia.pojo.PayOrder;
import com.platform.yijia.pojo.StationOilPrice;
import com.platform.yijia.service.AppUserInfoService;
import com.platform.yijia.service.CustomerGradeServices;
import com.platform.yijia.service.PayOrderService;
import com.platform.yijia.service.StationService;
import com.platform.yijia.utils.CodeMsg;
import com.platform.yijia.utils.ResultData;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import javax.naming.ldap.HasControls;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Controller
@RequestMapping("/demo")
public class PayOrderController {
    private static Logger logger =(Logger) LoggerFactory.getLogger(PayOrderController.class);
    @Resource
    private PayOrderService payOrderService;
    @Resource
    private AppUserInfoService appUserInfoService;
    @Resource
    private StationService stationService;
    @Resource
    private CustomerGradeServices customerGradeServices;

    //一键支付，生成订单
    @RequestMapping(value = "/AddPayOrderInfo", consumes = "application/json", method = RequestMethod.POST)
    @ResponseBody
    public String  AddPayOrderInfo(@RequestBody PayOrderRequest payOrderRequest){
        logger.info("获取订单前端传来参数openID："+ payOrderRequest.getOpenId());
        logger.info("获取订单前端传来参数userType："+ payOrderRequest.getUserType());
        Gson gson =new Gson();
        ResultData resultData =null;
        //根据token 和登录类型 身份验证
        AppUserInfo appUserInfo =new AppUserInfo();
        //订单类型 是小程序还是公众号
        appUserInfo.setUserType(payOrderRequest.getUserType());
        if(payOrderRequest.getUserType().equals("1")){
            //1 是公众号
            appUserInfo.setBlogOpenid(payOrderRequest.getOpenId());
        }else if(payOrderRequest.getUserType().equals("2")){
            //2是小程序
            appUserInfo.setMinaOpenid(payOrderRequest.getOpenId());
        }
        List<AppUserInfo> appUserInfoList =appUserInfoService.Authentication(appUserInfo);
        //判断登录人员是否验证成功
        if(appUserInfoList!=null&&appUserInfoList.size()>0){
            PayOrder payOrder=new PayOrder();
            payOrder.setConsumer(appUserInfoList.get(0).getBlogNickName()); //昵称
            payOrder.setConsumerId(appUserInfoList.get(0).getUserId()); //用户id
            //添加订单信息，返回订单号
            String ordNo =payOrderService.AddPayOrderInfo(payOrderRequest, payOrder);
            if(ordNo.equals("0")){
                resultData=ResultData.error(CodeMsg.CHECK_DATA);
            }else if(ordNo.equals("1")){
                resultData=ResultData.error(CodeMsg.PARAMS_NUll);
            }else {
                resultData=ResultData.success(ordNo);
            }
            //接口修订
//            Map<String, Object> stringObjectMap = payOrderService.AddPayOrderInfoNew(payOrderRequest, payOrder);
//            if(stringObjectMap.containsKey("balanceNoEnough")){
//                resultData=ResultData.error(CodeMsg.BALANCE_NOT_EENOUGH);
//            }else {
//                resultData = ResultData.success(stringObjectMap);
//            }
        }else{
            //登录人验证失败
            resultData=ResultData.error(CodeMsg.USER_NOT_EXSIST);
        }
        return gson.toJson(resultData);
    }

    /***
     * 计算订单金额
     * @return
     */
    @RequestMapping(value = "/calculateAmt", consumes = "application/json", method = RequestMethod.POST)
    @ResponseBody
    public String calculateAmt(@RequestBody PayOrderRequest payOrderRequest){
        Gson gson =new Gson();
        ResultData resultData =null;
        if(payOrderRequest.getReceivableAmt() !=null){  //应收金额
            Double receivableAmt = payOrderRequest.getReceivableAmt();  //应收金额 用户输入的金额
            Integer stationId = payOrderRequest.getStationId();         //油站ID
            String oilGun = payOrderRequest.getOilGun();                //油枪号
            String oilName = payOrderRequest.getOilName();          //油品名称
            String openId = payOrderRequest.getOpenId();        //用户的openId
            String mobilePhone = payOrderRequest.getCustomerPhone();        //客户手机号

            //获取油品价格
            StationOilPrice stationOilPrice = new StationOilPrice();
            stationOilPrice.setStationId(stationId);
            stationOilPrice.setOilName(oilName);
            StationOilPrice oilPriceInfo = stationService.getStationOilPrice(stationOilPrice);
            String oilPrice = oilPriceInfo.getOilPrice();
            BigDecimal oilLiters = null;
            BigDecimal discountAmt =null;
            BigDecimal discountPrice = null;
            BigDecimal amt = null;

            Map<String ,Object> resultMap = new HashMap<>();
            resultMap.put("oilPrice", oilPrice);        //油品价格
            resultMap.put("receivableAmt", receivableAmt);        //应收价格

            //该油站的优惠方式
            String stationDiscountWay = stationService.getStationDiscountWay(stationId.toString());
            Map<String ,Object> params = new HashMap<String, Object>();
            params.put("stationId", stationId);
            params.put("oilName", oilName);
            BigDecimal receivableAmt_b = new BigDecimal(receivableAmt); //传入的应收金额
            switch (stationDiscountWay){
                case "1":       //1.等级直降
                    //查询等级直降的优惠方式
                    params.put("mobilePhone", mobilePhone);
                    params.put("minaOpenid", openId);
                    //查询客户等级信息
                    List<Map<String, Object>> customerGradeInfo = customerGradeServices.getCustomerGradeInfo(params);
                    if(customerGradeInfo != null && customerGradeInfo.size() >0){   //已存在客户等级信息
                        //使用客户已存在的等级计算优惠
                        oilLiters = receivableAmt_b.divide(new BigDecimal(oilPrice),2, BigDecimal.ROUND_HALF_UP);
                        discountAmt = oilLiters.multiply(new BigDecimal(customerGradeInfo.get(0).get("gasoilDiscountLitre").toString()));
                        discountAmt = discountAmt.setScale(2, BigDecimal.ROUND_HALF_UP);
                        amt = receivableAmt_b.subtract(discountAmt);
                        discountPrice = amt.divide(oilLiters, 2, BigDecimal.ROUND_HALF_UP);
                        resultMap.put("discountAmt", discountAmt.doubleValue());
                        resultMap.put("amt", amt.doubleValue());
                        resultMap.put("oilLiters", oilLiters.doubleValue());
                        resultMap.put("discountPrice", discountPrice.doubleValue());
                    }else {
                        //不存在客户等级信息时，查询油站的客户等级直降优惠（查询结果根据会员成长值条件做升序查询）
                        List<Map> customerGradeList = customerGradeServices.getCustomerGradeList(params);
                        //如果存在
                        if(customerGradeList !=null && customerGradeList.size() >0){
                            //用户第一次购买 取等级最低初始会员的优惠条件计算
                            oilLiters = receivableAmt_b.divide(new BigDecimal(oilPrice), 2, BigDecimal.ROUND_HALF_UP);   //加油升数
                            discountAmt = oilLiters.multiply(new BigDecimal(customerGradeList.get(0).get("gasoilDiscountLitre").toString()));
                            discountAmt = discountAmt.setScale(2, BigDecimal.ROUND_HALF_UP);
                            amt = receivableAmt_b.subtract(discountAmt);
                            discountPrice = amt.divide(oilLiters, 2, BigDecimal.ROUND_HALF_UP);
                            resultMap.put("discountAmt", discountAmt.doubleValue());
                            resultMap.put("amt", amt.doubleValue());
                            resultMap.put("oilLiters", oilLiters.doubleValue());
                            resultMap.put("discountPrice", discountPrice.doubleValue());
                        }else {
                            //不存在
                            resultMap.put("discountAmt", 0);
                            resultMap.put("amt", receivableAmt.doubleValue());
                            oilLiters = new BigDecimal(receivableAmt).divide(new BigDecimal(oilPrice), 2, BigDecimal.ROUND_HALF_UP);
                            resultMap.put("oilLiters", oilLiters.doubleValue());
                            resultMap.put("discountPrice", oilPrice);
                        }
                    }
                    break;
                case "2":       //2.满减
                    params.put("discountPlanType", "1");
                    //营销方案
                    List<Map> customerMarkertPlanList_1 = customerGradeServices.getCustomerMarkertPlanList(params);
                    //有优惠规则
                    if(customerMarkertPlanList_1 !=null && customerMarkertPlanList_1.size() >0){
                        boolean flag = true;
                        for (Map m : customerMarkertPlanList_1){
                            //如果当前应收金额大于条件金额
                            BigDecimal discountAmt_b = new BigDecimal(m.get("discountAmt").toString());   //条件金额
                            if(receivableAmt_b.compareTo(discountAmt_b) ==1 || receivableAmt_b.compareTo(discountAmt_b) ==0){
                                BigDecimal gasoilDiscountAmt_b = new BigDecimal(m.get("gasoilDiscountAmt").toString());   //每满多少优惠多少的金额
                                discountAmt = receivableAmt_b.divide(discountAmt_b, 2, BigDecimal.ROUND_HALF_UP).multiply(gasoilDiscountAmt_b);  //优惠金额
                                discountAmt = discountAmt.setScale(2, BigDecimal.ROUND_HALF_UP);
                                amt = receivableAmt_b.subtract(discountAmt);   //实收金额
                                oilLiters = receivableAmt_b.divide(new BigDecimal(oilPrice),2, BigDecimal.ROUND_HALF_UP);   //加油升数
                                discountPrice = amt.divide(oilLiters);        //优惠价格
                                resultMap.put("discountAmt", discountAmt.doubleValue());
                                resultMap.put("amt", amt.doubleValue());
                                resultMap.put("oilLiters", oilLiters.doubleValue());
                                resultMap.put("discountPrice", discountPrice.doubleValue());
                                flag = false;
                                break;
                            }
                        }
                        if(flag){
                            resultMap.put("discountAmt", 0);
                            resultMap.put("amt", receivableAmt.doubleValue());
                            oilLiters = new BigDecimal(receivableAmt).divide(new BigDecimal(oilPrice),2, BigDecimal.ROUND_HALF_UP);
                            resultMap.put("oilLiters", oilLiters.doubleValue());
                            resultMap.put("discountPrice", oilPrice);
                        }
                    }else {
                        //没有优惠规则
                        resultMap.put("discountAmt", 0);
                        resultMap.put("amt", receivableAmt.doubleValue());
                        oilLiters = new BigDecimal(receivableAmt).divide(new BigDecimal(oilPrice), 2, BigDecimal.ROUND_HALF_UP);
                        resultMap.put("oilLiters", oilLiters.doubleValue());
                        resultMap.put("discountPrice", oilPrice);
                    }
                    break;
                case "3":       //3.立减
                    params.put("discountPlanType", "2");
                    //营销方案
                    List<Map> customerMarkertPlanList_2 = customerGradeServices.getCustomerMarkertPlanList(params);
                    if(customerMarkertPlanList_2 !=null && customerMarkertPlanList_2.size() >0){
                        boolean flag = true;
                        for (Map m : customerMarkertPlanList_2){
                            //如果当前应收金额大于条件金额
                            BigDecimal discountAmt_b = new BigDecimal(m.get("discountAmt").toString());   //条件金额
                            if(receivableAmt_b.compareTo(discountAmt_b) ==1 || receivableAmt_b.compareTo(discountAmt_b) ==0){
                                BigDecimal gasoilDiscountAmt_b = new BigDecimal(m.get("gasoilDiscountAmt").toString());   //每满多少优惠多少的金额
                                discountAmt = gasoilDiscountAmt_b;  //优惠金额
                                amt = receivableAmt_b.subtract(gasoilDiscountAmt_b);   //实收金额
                                oilLiters = receivableAmt_b.divide(new BigDecimal(oilPrice), 2, BigDecimal.ROUND_HALF_UP);   //加油升数
                                discountPrice = amt.divide(oilLiters);        //优惠价格
                                resultMap.put("discountAmt", discountAmt.doubleValue());
                                resultMap.put("amt", amt.doubleValue());
                                resultMap.put("oilLiters", oilLiters.doubleValue());
                                resultMap.put("discountPrice", discountPrice.doubleValue());
                                flag = false;
                                break;
                            }
                        }
                        if(flag){   //用户加油金额不符合优惠条件
                            resultMap.put("discountAmt", 0);
                            resultMap.put("amt", receivableAmt.doubleValue());
                            oilLiters = new BigDecimal(receivableAmt).divide(new BigDecimal(oilPrice), 2, BigDecimal.ROUND_HALF_UP);
                            resultMap.put("oilLiters", oilLiters.doubleValue());
                            resultMap.put("discountPrice", oilPrice);
                        }
                    }else {
                        //没有优惠规则
                        resultMap.put("discountAmt", 0);
                        resultMap.put("amt", receivableAmt.doubleValue());
                        oilLiters = new BigDecimal(receivableAmt).divide(new BigDecimal(oilPrice), 2, BigDecimal.ROUND_HALF_UP);
                        resultMap.put("oilLiters", oilLiters.doubleValue());
                        resultMap.put("discountPrice", oilPrice);
                    }
                    break;
                case "4":       //4.独立直降
                    params.put("discountPlanType", "3");
                    //营销方案
                    List<Map> customerMarkertPlanList_3 = customerGradeServices.getCustomerMarkertPlanList(params);
                    if(customerMarkertPlanList_3 !=null && customerMarkertPlanList_3.size() >0){
                        boolean flag= true;
                        for(Map m : customerMarkertPlanList_3){
                            BigDecimal discountAmt_b = new BigDecimal(m.get("discountAmt").toString());   //条件金额
                            BigDecimal gasoilDiscountAmt_b = new BigDecimal(m.get("gasoilDiscountAmt").toString());
                            oilLiters = receivableAmt_b.divide(new BigDecimal(oilPrice), 2, BigDecimal.ROUND_HALF_UP);   //加油升数
                            if(m.get("discountTerm").toString().equals("1")){   //按升数优惠
                                if(oilLiters.compareTo(discountAmt_b) ==1 || oilLiters.compareTo(discountAmt_b) ==0){
                                    //计算
                                    discountAmt = oilLiters.multiply(gasoilDiscountAmt_b);
                                    discountAmt = discountAmt.setScale(2, BigDecimal.ROUND_HALF_UP);
                                    amt = receivableAmt_b.subtract(discountAmt);
                                    discountPrice = amt.divide(oilLiters, 2, BigDecimal.ROUND_HALF_UP);        //优惠价格
                                    resultMap.put("discountAmt", discountAmt.doubleValue());
                                    resultMap.put("amt", amt.doubleValue());
                                    resultMap.put("oilLiters", oilLiters.doubleValue());
                                    resultMap.put("discountPrice", discountPrice.doubleValue());
                                    flag =false;
                                    break;
                                }
                            }else if(m.get("discountTerm").toString().equals("2")){     //按金额优惠
                                if(receivableAmt_b.compareTo(discountAmt_b) ==1 || receivableAmt_b.compareTo(discountAmt_b) ==0){
                                    discountAmt = oilLiters.multiply(gasoilDiscountAmt_b);
                                    discountAmt = discountAmt.setScale(2, BigDecimal.ROUND_HALF_UP);
                                    amt = receivableAmt_b.subtract(discountAmt);
                                    discountPrice = amt.divide(oilLiters, 2, BigDecimal.ROUND_HALF_UP);        //优惠价格
                                    resultMap.put("discountAmt", discountAmt.doubleValue());
                                    resultMap.put("amt", amt.doubleValue());
                                    resultMap.put("oilLiters", oilLiters.doubleValue());
                                    resultMap.put("discountPrice", discountPrice.doubleValue());
                                    flag =false;
                                    break;
                                }
                            }
                        }
                        if(flag){
                            resultMap.put("discountAmt", 0);
                            resultMap.put("amt", receivableAmt.doubleValue());
                            oilLiters = new BigDecimal(receivableAmt).divide(new BigDecimal(oilPrice), 2, BigDecimal.ROUND_HALF_UP);
                            resultMap.put("oilLiters", oilLiters.doubleValue());
                            resultMap.put("discountPrice", oilPrice);
                        }
                    }else {
                        //没有优惠规则
                        resultMap.put("discountAmt", 0);
                        resultMap.put("amt", receivableAmt.doubleValue());
                        oilLiters = receivableAmt_b.divide(new BigDecimal(oilPrice), 2, BigDecimal.ROUND_HALF_UP);
                        resultMap.put("oilLiters", oilLiters.doubleValue());
                        resultMap.put("discountPrice", oilPrice);
                    }
                    break;
            }
            resultData = ResultData.success(resultMap);
        }
        return gson.toJson(resultData);
    }



    /**
     * 根据订单号获取订单信息
     * produces="application/json;charset=UTF-8",consumes = "application/json",
     * @RequestBody String orderno
     * @param
     * @return
     */
    @RequestMapping(value = "/getPayOrderList", method = RequestMethod.GET)
    @ResponseBody
    public String getPayOrderList(@RequestParam String orderNo,String openId,String userType){
        Gson gson =new Gson();
        ResultData resultData =null;
        //根据token 和 usertype来判断用户是否存在
        boolean flag = appUserInfoService.selectAppUserInfo(openId,userType);
        if(flag){
            if(StringUtils.isNotBlank(orderNo)){
                PayOrderRequest payOrderRequest =new PayOrderRequest();
                payOrderRequest.setOrderNo(orderNo);
                List<PayOrder> payOrderList = payOrderService.getPayOrderList(payOrderRequest.getOrderNo());
                resultData=ResultData.success(payOrderList);
            }else {
                resultData=ResultData.error(CodeMsg.SEARCH_FAIL);
            }
        }else{
            resultData=ResultData.error(CodeMsg.USER_NOT_EXSIST);
        }
        return gson.toJson(resultData);
    }

    /*
     * 根据用户openId和用户类型userType查询用户订单
     * @param openId
     * @param userType
     * @return
     */
    @RequestMapping(value = "/getOrderListByOpenIdAndUserType", method = RequestMethod.GET)
    @ResponseBody
    public String getOrderListByOpenIdAndUserType(@RequestParam("openId") String openId, @RequestParam("userType") String userType){
        Gson gson =new Gson();
        ResultData resultData =null;
            if(StringUtils.isNotBlank(openId) && StringUtils.isNotBlank(userType)){
                AppUserInfo appUserInfo = new AppUserInfo();
                switch (userType){
                    case "1":
                        appUserInfo.setBlogOpenid(openId);
                        break;
                    case "2":
                        appUserInfo.setMinaOpenid(openId);
                        break;
                }
                List<PayOrder> payOrderList = payOrderService.getOrderListByOpenIdAndUserType(appUserInfo);
                resultData=ResultData.success(payOrderList);
            }else {
                resultData=ResultData.error(CodeMsg.SEARCH_FAIL);
            }

        return gson.toJson(resultData);
    }
}
